"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
const getPackageInfo_1 = require("../../../../utils/getPackageInfo");
const searchFilesInDirectory_1 = require("../../../../utils/searchFilesInDirectory");
const zipDirectory_1 = require("../../../../utils/zipDirectory");
const metadataFiles_1 = __importDefault(require("../../../../impl/metadata/metadataFiles"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const core_2 = require("@salesforce/core");
var path = require("path");
const spawn = require("child-process-promise").spawn;
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_picklist_generatepatch");
class Generatepatch extends command_1.SfdxCommand {
    async run() {
        //clean any existing temp sf powerkit source folder
        rimraf.sync("temp_sfpowerkit");
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        //
        // Getting Project config
        const project = await core_1.SfdxProject.resolve();
        const projectJson = await project.retrieveSfdxProjectJson();
        //Retrieve the package
        let packageToBeUsed;
        if (this.flags.package)
            packageToBeUsed = getPackageInfo_1.getPackageInfo(projectJson, this.flags.package);
        else {
            packageToBeUsed = getPackageInfo_1.getDefaultPackageInfo(projectJson);
        }
        this.flags.apiversion = this.flags.apiversion || "47.0";
        //set objects directory
        let objectsDirPath;
        if (this.flags.objectsdir)
            objectsDirPath = this.flags.objectsdir;
        else {
            objectsDirPath = packageToBeUsed.path + `/main/default/objects/`;
        }
        let status = await this.generatePatchForCustomPicklistField(objectsDirPath, this.flags.fixstandardvalueset);
        if (!status)
            return 1;
        sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", core_2.LoggerLevel.INFO);
        if (this.flags.fixrecordtypes) {
            let status = await this.generatePatchForBusinessProcess(objectsDirPath);
            if (!status)
                return 1;
            status = await this.generatePatchForRecordTypes(objectsDirPath);
            if (!status)
                return 1;
        }
        if (this.flags.movestandardvalueset) {
            status = await this.generatePatchForStandardValuset(objectsDirPath);
            if (!status)
                return 1;
        }
        // sfdx project json file running force source command
        var sfdx_project_json = `{
        "packageDirectories": [
          {
            "path": "${packageToBeUsed.path}",
            "default": true
          }
        ],
        "namespace": "",
        "sourceApiVersion": "${this.flags.apiversion}"
      }`;
        fs_extra_1.default.outputFileSync("temp_sfpowerkit/sfdx-project.json", sfdx_project_json);
        //force ignore file to ignore custom metadata
        var forceIgnoreFile = `**__mdt/`;
        fs_extra_1.default.outputFileSync("temp_sfpowerkit/.forceignore", forceIgnoreFile);
        if (fs_extra_1.default.existsSync(path.resolve(`temp_sfpowerkit/${packageToBeUsed.path}`))) {
            //Convert to mdapi
            const args = [];
            args.push("force:source:convert");
            args.push("-r");
            args.push(`${packageToBeUsed.path}`);
            args.push("-d");
            args.push(`mdapi`);
            await spawn("sfdx", args, {
                stdio: "inherit",
                cwd: "temp_sfpowerkit"
            });
            //Generate zip file
            var zipFile = "temp_sfpowerkit/" + `${packageToBeUsed.package}` + "_picklist.zip";
            await zipDirectory_1.zipDirectory("temp_sfpowerkit/mdapi", zipFile);
            //Create Static Resource Directory if not exist
            let dir = packageToBeUsed.path + `/main/default/staticresources/`;
            if (!fs_extra_1.default.existsSync(dir)) {
                fs_extra_1.default.mkdirSync(dir);
            }
            fs_extra_1.default.copyFileSync(zipFile, packageToBeUsed.path +
                `/main/default/staticresources/${packageToBeUsed.package}_picklist.zip`);
            //Store it to static resources
            var metadata = `<?xml version="1.0" encoding="UTF-8"?>
        <StaticResource xmlns="http://soap.sforce.com/2006/04/metadata">
            <cacheControl>Public</cacheControl>
            <contentType>application/zip</contentType>
        </StaticResource>`;
            let targetmetadatapath = packageToBeUsed.path +
                `/main/default/staticresources/${packageToBeUsed.package}_picklist.resource-meta.xml`;
            sfpowerkit_1.SFPowerkit.log("Generating static resource file : " + `${targetmetadatapath}`, core_2.LoggerLevel.INFO);
            fs_extra_1.default.outputFileSync(targetmetadatapath, metadata);
            sfpowerkit_1.SFPowerkit.log(`Patch ${packageToBeUsed.package}_picklist generated successfully.`, core_2.LoggerLevel.INFO);
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`No picklist or recordtype found to create a Patch `, core_2.LoggerLevel.WARN);
        }
        //clean temp sf powerkit source folder
        rimraf.sync("temp_sfpowerkit");
        return 0;
    }
    async generatePatchForCustomPicklistField(objectsDirPath, isStandardValueSetToBeFixed) {
        if (isStandardValueSetToBeFixed) {
            this.ux
                .log(`Warning, your package source code will be modified to remove references to standard value set and will be 
      added into the patch`);
        }
        sfpowerkit_1.SFPowerkit.log("Scanning for fields of type picklist", core_2.LoggerLevel.INFO);
        //search picklist
        let customFieldsWithPicklist = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, "<type>Picklist</type>", ".xml");
        //search MultiselectPicklist
        let customFieldsWithMultiPicklist = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, "<type>MultiselectPicklist</type>", ".xml");
        if (customFieldsWithMultiPicklist &&
            customFieldsWithMultiPicklist.length > 0) {
            customFieldsWithPicklist = customFieldsWithPicklist.concat(customFieldsWithMultiPicklist);
        }
        if (customFieldsWithPicklist && customFieldsWithPicklist.length > 0) {
            sfpowerkit_1.SFPowerkit.log("Found " +
                `${customFieldsWithPicklist.length}` +
                " fields of type picklist", core_2.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log("Processing and adding the following fields to patch", core_2.LoggerLevel.INFO);
            let in_patch_count = 0;
            let modified_source_count = 0;
            for (const file of customFieldsWithPicklist) {
                const parser = new xml2js.Parser({ explicitArray: false });
                const parseString = util.promisify(parser.parseString);
                let field_metadata;
                try {
                    field_metadata = await parseString(fs_extra_1.default.readFileSync(path.resolve(file)));
                }
                catch (e) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to parse file ${file} due to ${e}`, core_2.LoggerLevel.FATAL);
                    return Promise.reject(e);
                }
                let controllingField = (field_metadata.CustomField.valueSet || {}).controllingField;
                //Skip mdt pick fields as its non upgradeable if owned by a package
                if (!String(file).includes("__mdt\\fields")) {
                    //A controlling field with a standarad value set, patch source and copy original to patch
                    if (controllingField != undefined &&
                        !controllingField.endsWith("__c")) {
                        if (isStandardValueSetToBeFixed) {
                            in_patch_count++;
                            sfpowerkit_1.SFPowerkit.log("Copied Original to Patch:         " + file, core_2.LoggerLevel.INFO);
                            metadataFiles_1.default.copyFile(file, "temp_sfpowerkit");
                            modified_source_count++;
                            let builder = new xml2js.Builder();
                            delete field_metadata.CustomField.valueSet.controllingField;
                            delete field_metadata.CustomField.valueSet.valueSettings;
                            var xml = builder.buildObject(field_metadata);
                            fs_extra_1.default.writeFileSync(file, xml);
                            sfpowerkit_1.SFPowerkit.log("Modified Original in Packaging:         " + file, core_2.LoggerLevel.INFO);
                        }
                    }
                    else {
                        in_patch_count++;
                        sfpowerkit_1.SFPowerkit.log("Copied Original to Patch:         " + file, core_2.LoggerLevel.INFO);
                        metadataFiles_1.default.copyFile(file, "temp_sfpowerkit");
                    }
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Added  ${in_patch_count} fields of field type picklist into patch after'removing fields picklist fields in cmdt objects`, core_2.LoggerLevel.INFO);
            if (this.flags.fixstandardvalueset)
                sfpowerkit_1.SFPowerkit.log(`Modified  ${modified_source_count} fields of field type picklist that have standard value sets as controlling types in packaging folder`, core_2.LoggerLevel.INFO);
        }
        return Promise.resolve(true);
    }
    async generatePatchForRecordTypes(objectsDirPath) {
        this.ux
            .log(`Warning, your package source code will be modified to remove references to standard value set and the orginal source code
    will be  added into the patch`);
        sfpowerkit_1.SFPowerkit.log("Scanning for recordtypes", core_2.LoggerLevel.INFO);
        let recordTypes = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, '<RecordType xmlns="http://soap.sforce.com/2006/04/metadata">', ".xml");
        if (recordTypes && recordTypes.length > 0) {
            sfpowerkit_1.SFPowerkit.log("Found " + `${recordTypes.length}` + " RecordTypes", core_2.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log("Processing and adding the following fields to patch", core_2.LoggerLevel.INFO);
            let in_patch_count = 0;
            let modified_source_count = 0;
            for (const file of recordTypes) {
                in_patch_count++;
                modified_source_count++;
                const parser = new xml2js.Parser({ explicitArray: false });
                const parseString = util.promisify(parser.parseString);
                let recordtype_metadata;
                try {
                    recordtype_metadata = await parseString(fs_extra_1.default.readFileSync(path.resolve(file)));
                }
                catch (e) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to parse file ${file} due to ${e}`, core_2.LoggerLevel.FATAL);
                    return false;
                }
                sfpowerkit_1.SFPowerkit.log("Copied Original to Patch:         " + file, core_2.LoggerLevel.INFO);
                metadataFiles_1.default.copyFile(file, "temp_sfpowerkit");
                delete recordtype_metadata.RecordType.picklistValues;
                let builder = new xml2js.Builder();
                var xml = builder.buildObject(recordtype_metadata);
                fs_extra_1.default.writeFileSync(file, xml);
                sfpowerkit_1.SFPowerkit.log("Modified Original in Packaging:         " + file, core_2.LoggerLevel.INFO);
            }
            sfpowerkit_1.SFPowerkit.log(`Added  ${in_patch_count}  RecordType to patch`, core_2.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log(`Modified  ${modified_source_count} RecordTypes in packaging folder`, core_2.LoggerLevel.INFO);
        }
        sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", core_2.LoggerLevel.INFO);
        return true;
    }
    async generatePatchForBusinessProcess(objectsDirPath) {
        let patch_value = {
            fullName: "New",
            default: "true"
        };
        this.ux
            .log(`Warning, your package source code will be modified to remove references to standard value set and the orginal source code
    will be  added into the patch`);
        sfpowerkit_1.SFPowerkit.log("Scanning for BusinessProcess", core_2.LoggerLevel.INFO);
        let businessProcess = searchFilesInDirectory_1.searchFilesInDirectory(objectsDirPath, '<BusinessProcess xmlns="http://soap.sforce.com/2006/04/metadata">', ".xml");
        if (businessProcess && businessProcess.length > 0) {
            sfpowerkit_1.SFPowerkit.log("Found " + `${businessProcess.length}` + " BusinessProcess", core_2.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log("Processing and adding the following fields to patch", core_2.LoggerLevel.INFO);
            let in_patch_count = 0;
            let modified_source_count = 0;
            for (const file of businessProcess) {
                in_patch_count++;
                modified_source_count++;
                const parser = new xml2js.Parser({ explicitArray: false });
                const parseString = util.promisify(parser.parseString);
                let businessProcess_metadata;
                try {
                    businessProcess_metadata = await parseString(fs_extra_1.default.readFileSync(path.resolve(file)));
                }
                catch (e) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to parse file ${file} due to ${e}`, core_2.LoggerLevel.FATAL);
                    return false;
                }
                sfpowerkit_1.SFPowerkit.log("Copied Original to Patch:         " + file, core_2.LoggerLevel.INFO);
                metadataFiles_1.default.copyFile(file, "temp_sfpowerkit");
                businessProcess_metadata.BusinessProcess.values = [];
                businessProcess_metadata.BusinessProcess.values.push(patch_value);
                let builder = new xml2js.Builder();
                var xml = builder.buildObject(businessProcess_metadata);
                fs_extra_1.default.writeFileSync(file, xml);
                sfpowerkit_1.SFPowerkit.log("Modified Original in Packaging:         " + file, core_2.LoggerLevel.INFO);
            }
            sfpowerkit_1.SFPowerkit.log(`Added  ${in_patch_count}  BusinessProcess to patch`, core_2.LoggerLevel.INFO);
            sfpowerkit_1.SFPowerkit.log(`Modified  ${modified_source_count} BusinessProcess in packaging folder`, core_2.LoggerLevel.INFO);
        }
        sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", core_2.LoggerLevel.INFO);
        return true;
    }
    async generatePatchForStandardValuset(objectsDirPath) {
        this.ux
            .log(`Warning, your package source code will be modified to remove standard valueset. The modified source will be 
    added into the patch`);
        let standardValueSetPath = objectsDirPath.replace("objects", "");
        if (standardValueSetPath.includes("//")) {
            standardValueSetPath = standardValueSetPath.replace("//", "/");
        }
        standardValueSetPath = standardValueSetPath + "standardValueSets/";
        if (fs_extra_1.default.existsSync(path.resolve(standardValueSetPath))) {
            let standardValueSets = searchFilesInDirectory_1.searchFilesInDirectory(standardValueSetPath, '<StandardValueSet xmlns="http://soap.sforce.com/2006/04/metadata">', ".xml");
            if (standardValueSets.length > 0) {
                sfpowerkit_1.SFPowerkit.log(`Found ${standardValueSets.length} Standard valueset in ${standardValueSetPath}`, core_2.LoggerLevel.INFO);
                for (const file of standardValueSets) {
                    sfpowerkit_1.SFPowerkit.log("Copied Original to Patch:         " + file, core_2.LoggerLevel.INFO);
                    metadataFiles_1.default.copyFile(file, "temp_sfpowerkit");
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Removing ${standardValueSetPath} from source`, core_2.LoggerLevel.INFO);
            rimraf.sync(standardValueSetPath);
            sfpowerkit_1.SFPowerkit.log("--------------------------------------------------------------------------------", core_2.LoggerLevel.INFO);
        }
        return Promise.resolve(true);
    }
}
exports.default = Generatepatch;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Generatepatch.requiresProject = true;
Generatepatch.description = messages.getMessage("commandDescription");
Generatepatch.examples = [
    `$ sfdx sfpowerkit:source:picklist:generatepatch -p sfpowerkit_test -d force-app/main/default/objects/ -f`
];
Generatepatch.flagsConfig = {
    package: command_1.flags.string({
        required: false,
        char: "p",
        description: messages.getMessage("packageFlagDescription")
    }),
    objectsdir: command_1.flags.string({
        required: false,
        char: "d",
        description: messages.getMessage("objectDirFlagDescription")
    }),
    fixstandardvalueset: command_1.flags.boolean({
        required: false,
        char: "f",
        description: messages.getMessage("fixStandardValueSetDescription")
    }),
    fixrecordtypes: command_1.flags.boolean({
        required: false,
        char: "r",
        description: messages.getMessage("fixRecordTypes")
    }),
    movestandardvalueset: command_1.flags.boolean({
        required: false,
        char: "m",
        description: messages.getMessage("movestandardvalueSetDescription")
    }),
    apiversion: command_1.flags.builtin({
        description: messages.getMessage("apiversion")
    }),
    loglevel: command_1.flags.enum({
        description: messages.getMessage("loglevel"),
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
//# sourceMappingURL=generatepatch.js.map