"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const metadataInfo_1 = require("./metadataInfo");
const fileutils_1 = __importDefault(require("../../utils/fileutils"));
const _ = __importStar(require("lodash"));
const ignore_1 = __importDefault(require("ignore"));
const fs = __importStar(require("fs"));
const glob = __importStar(require("glob"));
const sfpowerkit_1 = require("../../sfpowerkit");
const core_1 = require("@salesforce/core");
class MetadataFiles {
    constructor() {
        if (fs.existsSync(".forceignore")) {
            this.forceignore = ignore_1.default().add(fs.readFileSync(".forceignore", "utf8").toString());
        }
        else {
            this.forceignore = ignore_1.default();
        }
    }
    static getFullApiName(fileName) {
        let fullName = "";
        let metadateType = metadataInfo_1.MetadataInfo.getMetadataName(fileName);
        let splitFilepath = fileName.split(path.sep);
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(metadateType);
        if (isObjectChild) {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1].split(".")[0];
            fullName = objectName.concat("." + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1].split(".")[0];
        }
        return fullName;
    }
    static getFullApiNameWithExtension(fileName) {
        let fullName = "";
        let metadateType = metadataInfo_1.MetadataInfo.getMetadataName(fileName);
        let splitFilepath = fileName.split(path.sep);
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(metadateType);
        if (isObjectChild) {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1];
            fullName = objectName.concat("." + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1];
        }
        return fullName;
    }
    static isCustomMetadata(filepath, name) {
        let result = true;
        let splitFilepath = filepath.split(path.sep);
        let componentName = splitFilepath[splitFilepath.length - 1];
        componentName = componentName.substring(0, componentName.indexOf("."));
        if (name === metadataInfo_1.METADATA_INFO.CustomField.xmlName ||
            name === metadataInfo_1.METADATA_INFO.CustomObject.xmlName) {
            //Custom Field or Custom Object
            result = componentName.endsWith("__c") || componentName.endsWith("__mdt");
        }
        return result;
    }
    static getMemberNameFromFilepath(filepath, name) {
        let member;
        let splitFilepath = filepath.split(path.sep);
        let lastIndex = splitFilepath.length - 1;
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(name);
        let metadataDescribe = metadataInfo_1.METADATA_INFO[name];
        if (isObjectChild) {
            let objectName = splitFilepath[lastIndex - 2];
            let fieldName = splitFilepath[lastIndex].split(".")[0];
            member = objectName.concat("." + fieldName);
        }
        else if (metadataDescribe.inFolder) {
            let baseName = metadataDescribe.directoryName;
            let baseIndex = filepath.indexOf(baseName) + baseName.length;
            let cmpPath = filepath.substring(baseIndex + 1); // add 1 to remove the path seperator
            cmpPath = cmpPath.substring(0, cmpPath.indexOf("."));
            member = cmpPath.replace(path.sep, "/");
        }
        else {
            member = splitFilepath[lastIndex].replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, "");
        }
        return member;
    }
    loadComponents(srcFolder, checkIgnore = true) {
        var metadataFiles = fileutils_1.default.getAllFilesSync(srcFolder);
        let keys = Object.keys(metadataInfo_1.METADATA_INFO);
        if (Array.isArray(metadataFiles) && metadataFiles.length > 0) {
            metadataFiles.forEach(metadataFile => {
                for (let i = 0; i < keys.length; i++) {
                    let match = false;
                    if (metadataFile.endsWith(metadataInfo_1.METADATA_INFO[keys[i]].sourceExtension)) {
                        match = true;
                    }
                    else if (metadataInfo_1.METADATA_INFO[keys[i]].inFolder &&
                        metadataFile.endsWith(metadataInfo_1.METADATA_INFO[keys[i]].folderExtension)) {
                        match = true;
                    }
                    if (match) {
                        if (_.isNil(metadataInfo_1.METADATA_INFO[keys[i]].files)) {
                            metadataInfo_1.METADATA_INFO[keys[i]].files = [];
                            metadataInfo_1.METADATA_INFO[keys[i]].components = [];
                        }
                        if (!checkIgnore || (checkIgnore && this.accepts(metadataFile))) {
                            metadataInfo_1.METADATA_INFO[keys[i]].files.push(metadataFile);
                            let name = fileutils_1.default.getFileNameWithoutExtension(metadataFile, metadataInfo_1.METADATA_INFO[keys[i]].sourceExtension);
                            if (metadataInfo_1.METADATA_INFO[keys[i]].isChildComponent) {
                                let fileParts = metadataFile.split(path.sep);
                                let parentName = fileParts[fileParts.length - 3];
                                name = parentName + "." + name;
                            }
                            metadataInfo_1.METADATA_INFO[keys[i]].components.push(name);
                        }
                        break;
                    }
                }
            });
        }
        else {
            keys.forEach(key => {
                if (_.isNil(metadataInfo_1.METADATA_INFO[key].files)) {
                    metadataInfo_1.METADATA_INFO[key].files = [];
                    metadataInfo_1.METADATA_INFO[key].components = [];
                }
            });
        }
    }
    //Check if a component is accepted by forceignore.
    accepts(filePath) {
        return !this.forceignore.ignores(path.relative(process.cwd(), filePath));
    }
    async isInModuleFolder(filePath) {
        const packageDirectories = await sfpowerkit_1.SFPowerkit.getProjectDirectories();
        if (!packageDirectories || packageDirectories.length == 0) {
            return false;
        }
        const moduleFolder = packageDirectories.find(packageFolder => {
            let packageFolderNormalized = path.relative("", packageFolder);
            return filePath.startsWith(packageFolderNormalized);
        });
        return moduleFolder !== undefined;
    }
    /**
     * Copy a file to an outpu directory. If the filePath is a Metadata file Path,
     * All the metadata requirement are also copied. For example MyApexClass.cls-meta.xml will also copy MyApexClass.cls.
     * Enforcing the .forceignore to ignire file ignored in the project.
     * @param filePath
     * @param outputFolder
     */
    static copyFile(filePath, outputFolder) {
        sfpowerkit_1.SFPowerkit.log(`Copying file ${filePath} from file system to ${outputFolder}`, core_1.LoggerLevel.DEBUG);
        const LWC_IGNORE_FILES = ["jsconfig.json", ".eslintrc.json"];
        const pairStatResources = metadataInfo_1.METADATA_INFO.StaticResource.directoryName;
        const pairStatResourcesRegExp = new RegExp(pairStatResources);
        const pairAuaraRegExp = new RegExp(metadataInfo_1.METADATA_INFO.AuraDefinitionBundle.directoryName);
        let copyOutputFolder = outputFolder;
        if (!fs.existsSync(filePath)) {
            return;
        }
        let exists = fs.existsSync(path.join(outputFolder, filePath));
        if (exists) {
            return;
        }
        if (filePath.startsWith(".")) {
            let parts = path.parse(filePath);
            if (parts.dir === "") {
                fs.copyFileSync(filePath, path.join(outputFolder, filePath));
                return;
            }
        }
        let fileName = path.parse(filePath).base;
        //exclude lwc ignored files
        if (LWC_IGNORE_FILES.includes(fileName)) {
            return;
        }
        let filePathParts = filePath.split(path.sep);
        if (fs.existsSync(outputFolder) == false) {
            fs.mkdirSync(outputFolder);
        }
        // Create folder structure
        for (let i = 0; i < filePathParts.length - 1; i++) {
            let folder = filePathParts[i].replace('"', "");
            outputFolder = path.join(outputFolder, folder);
            if (fs.existsSync(outputFolder) == false) {
                fs.mkdirSync(outputFolder);
            }
        }
        // Copy all file with same base name
        let associatedFilePattern = "";
        if (metadataInfo_1.SOURCE_EXTENSION_REGEX.test(filePath)) {
            associatedFilePattern = filePath.replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, ".*");
        }
        else {
            let extension = path.parse(filePath).ext;
            associatedFilePattern = filePath.replace(extension, ".*");
        }
        let files = glob.sync(associatedFilePattern);
        for (let i = 0; i < files.length; i++) {
            if (fs.lstatSync(files[i]).isDirectory() == false) {
                let oneFilePath = path.join(".", files[i]);
                let oneFilePathParts = oneFilePath.split(path.sep);
                fileName = oneFilePathParts[oneFilePathParts.length - 1];
                let outputPath = path.join(outputFolder, fileName);
                fs.copyFileSync(files[i], outputPath);
            }
        }
        // Hadle ObjectTranslations
        // If a file fieldTranslation is copied, make sure the ObjectTranslation File is also copied
        if (filePath.endsWith("Translation-meta.xml") &&
            filePath.indexOf("globalValueSet") < 0) {
            let parentFolder = filePathParts[filePathParts.length - 2];
            let objectTranslation = parentFolder + metadataInfo_1.METADATA_INFO.CustomObjectTranslation.sourceExtension;
            let outputPath = path.join(outputFolder, objectTranslation);
            let sourceFile = filePath.replace(fileName, objectTranslation);
            if (fs.existsSync(sourceFile) == true) {
                fs.copyFileSync(sourceFile, outputPath);
            }
        }
        //FOR STATIC RESOURCES - WHERE THE CORRESPONDING DIRECTORY + THE ROOT META FILE HAS TO BE INCLUDED
        if (pairStatResourcesRegExp.test(filePath)) {
            outputFolder = path.join(".", copyOutputFolder);
            let srcFolder = ".";
            let staticRecourceRoot = "";
            let resourceFile = "";
            for (let i = 0; i < filePathParts.length; i++) {
                outputFolder = path.join(outputFolder, filePathParts[i]);
                srcFolder = path.join(srcFolder, filePathParts[i]);
                if (filePathParts[i] === metadataInfo_1.METADATA_INFO.StaticResource.directoryName) {
                    let fileOrDirname = filePathParts[i + 1];
                    let fileOrDirnameParts = fileOrDirname.split(".");
                    srcFolder = path.join(srcFolder, fileOrDirnameParts[0]);
                    outputFolder = path.join(outputFolder, fileOrDirnameParts[0]);
                    resourceFile =
                        srcFolder + metadataInfo_1.METADATA_INFO.StaticResource.sourceExtension;
                    metadataInfo_1.METADATA_INFO.StaticResource.sourceExtension;
                    staticRecourceRoot =
                        outputFolder + metadataInfo_1.METADATA_INFO.StaticResource.sourceExtension;
                    if (fs.existsSync(srcFolder)) {
                        if (fs.existsSync(outputFolder) == false) {
                            fs.mkdirSync(outputFolder);
                        }
                    }
                    break;
                }
            }
            if (fs.existsSync(srcFolder)) {
                fileutils_1.default.copyRecursiveSync(srcFolder, outputFolder);
            }
            if (fs.existsSync(resourceFile)) {
                fs.copyFileSync(resourceFile, staticRecourceRoot);
            }
        }
        //FOR AURA components and LWC components
        if (pairAuaraRegExp.test(filePath)) {
            outputFolder = path.join(".", copyOutputFolder);
            let srcFolder = ".";
            for (let i = 0; i < filePathParts.length; i++) {
                outputFolder = path.join(outputFolder, filePathParts[i]);
                srcFolder = path.join(srcFolder, filePathParts[i]);
                if (filePathParts[i] === "aura" || filePathParts[i] === "lwc") {
                    let fileOrDirname = filePathParts[i + 1];
                    let fileOrDirnameParts = fileOrDirname.split(".");
                    srcFolder = path.join(srcFolder, fileOrDirnameParts[0]);
                    outputFolder = path.join(outputFolder, fileOrDirnameParts[0]);
                    if (fs.existsSync(srcFolder)) {
                        if (fs.existsSync(outputFolder) == false) {
                            fs.mkdirSync(outputFolder);
                        }
                    }
                    break;
                }
            }
            if (fs.existsSync(srcFolder)) {
                fileutils_1.default.copyRecursiveSync(srcFolder, outputFolder);
            }
        }
    }
}
exports.default = MetadataFiles;
MetadataFiles.sourceOnly = false;
//# sourceMappingURL=metadataFiles.js.map