"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const _ = __importStar(require("lodash"));
const simplegit = require("simple-git/promise");
const metadataFiles_1 = __importDefault(require("../../../impl/metadata/metadataFiles"));
const metadataInfo_1 = require("../../../impl/metadata/metadataInfo");
const metadataInfo_2 = require("../../../impl/metadata/metadataInfo");
const sfpowerkit_1 = require("../../../sfpowerkit");
const core_1 = require("@salesforce/core");
const git = simplegit();
class DiffUtil {
    static async isFormulaField(diffFile) {
        let content = await git.show(["--raw", diffFile.revisionFrom]);
        let result = content.includes("<formula>");
        return result;
    }
    static async fetchFileListRevisionTo(revisionTo) {
        sfpowerkit_1.SFPowerkit.log("Fetching file list from target revision " + revisionTo, core_1.LoggerLevel.INFO);
        DiffUtil.gitTreeRevisionTo = [];
        let revisionTree = await git.raw(["ls-tree", "-r", revisionTo]);
        const sepRegex = /\n|\r/;
        let lines = revisionTree.split(sepRegex);
        for (let i = 0; i < lines.length; i++) {
            if (lines[i] === "")
                continue;
            let fields = lines[i].split(/\t/);
            let pathStr = fields[1];
            let revisionSha = fields[0].split(/\s/)[2];
            let oneFIle = {
                revision: revisionSha,
                path: path.join(".", pathStr)
            };
            DiffUtil.gitTreeRevisionTo.push(oneFIle);
        }
        return DiffUtil.gitTreeRevisionTo;
    }
    static async getRelativeFiles(filePath) {
        let relativeFiles = [];
        let filePathParts = filePath.split(path.sep);
        const statResourcesRegExp = new RegExp(metadataInfo_2.METADATA_INFO.StaticResource.directoryName);
        const auraRegExp = new RegExp(metadataInfo_2.METADATA_INFO.AuraDefinitionBundle.directoryName);
        const lwcRegExp = new RegExp(metadataInfo_2.METADATA_INFO.LightningComponentBundle.directoryName);
        if (filePath.endsWith("Translation-meta.xml") &&
            filePath.indexOf("globalValueSet") < 0) {
            let parentFolder = filePathParts[filePathParts.length - 2];
            let objectTranslation = parentFolder + metadataInfo_2.METADATA_INFO.CustomObjectTranslation.sourceExtension;
            DiffUtil.gitTreeRevisionTo.forEach(file => {
                if (file.path === filePath || file.path === objectTranslation) {
                    relativeFiles.push(file);
                }
            });
        }
        else if (statResourcesRegExp.test(filePath) ||
            auraRegExp.test(filePath) ||
            lwcRegExp.test(filePath)) {
            // handle static recources
            let baseFile = "";
            for (let i = 0; i < filePathParts.length; i++) {
                baseFile = path.join(baseFile, filePathParts[i]);
                if (filePathParts[i] === metadataInfo_2.METADATA_INFO.StaticResource.directoryName ||
                    filePathParts[i] ===
                        metadataInfo_2.METADATA_INFO.AuraDefinitionBundle.directoryName ||
                    filePathParts[i] ===
                        metadataInfo_2.METADATA_INFO.LightningComponentBundle.directoryName) {
                    let fileOrDirname = filePathParts[i + 1];
                    if (metadataInfo_1.SOURCE_EXTENSION_REGEX.test(fileOrDirname)) {
                        fileOrDirname = fileOrDirname.replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, "");
                    }
                    else {
                        let extension = path.parse(fileOrDirname).ext;
                        fileOrDirname = fileOrDirname.replace(extension, "");
                    }
                    baseFile = path.join(baseFile, fileOrDirname);
                    break;
                }
            }
            DiffUtil.gitTreeRevisionTo.forEach(file => {
                let fileToCompare = file.path;
                if (fileToCompare.startsWith(baseFile)) {
                    relativeFiles.push(file);
                }
            });
        }
        else {
            let baseFile = filePath;
            if (metadataInfo_1.SOURCE_EXTENSION_REGEX.test(filePath)) {
                baseFile = filePath.replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, "");
            }
            else {
                let extension = path.parse(filePath).ext;
                baseFile = filePath.replace(extension, "");
            }
            DiffUtil.gitTreeRevisionTo.forEach(file => {
                let fileToCompare = file.path;
                if (metadataInfo_1.SOURCE_EXTENSION_REGEX.test(fileToCompare)) {
                    fileToCompare = fileToCompare.replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, "");
                }
                else {
                    let extension = path.parse(fileToCompare).ext;
                    fileToCompare = fileToCompare.replace(extension, "");
                }
                if (baseFile === fileToCompare) {
                    relativeFiles.push(file);
                }
            });
        }
        return relativeFiles;
    }
    static async copyFile(filePath, outputFolder) {
        sfpowerkit_1.SFPowerkit.log(`Copying file ${filePath} from git to ${outputFolder}`, core_1.LoggerLevel.INFO);
        if (fs.existsSync(path.join(outputFolder, filePath))) {
            sfpowerkit_1.SFPowerkit.log(`File ${filePath}  already in output folder. `, core_1.LoggerLevel.TRACE);
            return;
        }
        let gitFiles = await DiffUtil.getRelativeFiles(filePath);
        let copyOutputFolder = outputFolder;
        for (let i = 0; i < gitFiles.length; i++) {
            outputFolder = copyOutputFolder;
            let gitFile = gitFiles[i];
            sfpowerkit_1.SFPowerkit.log(`Associated file ${i}: ${gitFile.path}  Revision: ${gitFile.revision}`, core_1.LoggerLevel.TRACE);
            let outputPath = path.join(outputFolder, gitFile.path);
            let filePathParts = gitFile.path.split(path.sep);
            if (fs.existsSync(outputFolder) == false) {
                fs.mkdirSync(outputFolder);
            }
            // Create folder structure
            for (let i = 0; i < filePathParts.length - 1; i++) {
                let folder = filePathParts[i].replace('"', "");
                outputFolder = path.join(outputFolder, folder);
                if (fs.existsSync(outputFolder) == false) {
                    fs.mkdirSync(outputFolder);
                }
            }
            let fileContent = await git.show(["--raw", gitFile.revision]);
            fs.writeFileSync(outputPath, fileContent);
        }
    }
    static async parseContent(fileContents) {
        const statusRegEx = /\sA\t|\sM\t|\sD\t/;
        const renamedRegEx = /\sR[0-9]{3}\t|\sC[0-9]{3}\t/;
        const tabRegEx = /\t/;
        const deletedFileRegEx = new RegExp(/\sD\t/);
        const lineBreakRegEx = /\r?\n|\r|( $)/;
        let metadataFiles = new metadataFiles_1.default();
        var diffFile = {
            deleted: [],
            addedEdited: []
        };
        for (var i = 0; i < fileContents.length; i++) {
            if (statusRegEx.test(fileContents[i])) {
                var lineParts = fileContents[i].split(statusRegEx);
                var finalPath = path.join(".", lineParts[1].replace(lineBreakRegEx, ""));
                finalPath = finalPath.trim();
                finalPath = finalPath.replace("\\303\\251", "é");
                if (!(await metadataFiles.isInModuleFolder(finalPath))) {
                    continue;
                }
                if (!metadataFiles.accepts(finalPath)) {
                    continue;
                }
                let revisionPart = lineParts[0].split(/\t|\s/);
                if (deletedFileRegEx.test(fileContents[i])) {
                    //Deleted
                    diffFile.deleted.push({
                        revisionFrom: revisionPart[2].substring(0, 9),
                        revisionTo: revisionPart[3].substring(0, 9),
                        path: finalPath
                    });
                }
                else {
                    // Added or edited
                    diffFile.addedEdited.push({
                        revisionFrom: revisionPart[2].substring(0, 9),
                        revisionTo: revisionPart[3].substring(0, 9),
                        path: finalPath
                    });
                }
            }
            else if (renamedRegEx.test(fileContents[i])) {
                var lineParts = fileContents[i].split(renamedRegEx);
                var pathsParts = path.join(".", lineParts[1].trim());
                pathsParts = pathsParts.replace("\\303\\251", "é");
                let revisionPart = lineParts[0].split(/\t|\s/);
                var paths = pathsParts.split(tabRegEx);
                if (!(await metadataFiles.isInModuleFolder(finalPath))) {
                    continue;
                }
                if (!metadataFiles.accepts(paths[0].trim())) {
                    continue;
                }
                diffFile.addedEdited.push({
                    revisionFrom: "000000000",
                    revisionTo: revisionPart[3],
                    renamedPath: paths[0].trim(),
                    path: paths[1].trim()
                });
                //allow deletion of renamed components
                diffFile.deleted.push({
                    revisionFrom: revisionPart[2],
                    revisionTo: "000000000",
                    path: paths[0].trim()
                });
            }
        }
        return diffFile;
    }
    static getChangedOrAdded(list1, list2, key) {
        let result = {
            addedEdited: [],
            deleted: []
        };
        //Ensure array
        if (!_.isNil(list1) && !Array.isArray(list1)) {
            list1 = [list1];
        }
        if (!_.isNil(list2) && !Array.isArray(list2)) {
            list2 = [list2];
        }
        if (_.isNil(list1) && !_.isNil(list2) && list2.length > 0) {
            result.addedEdited.push(...list2);
        }
        if (_.isNil(list2) && !_.isNil(list1) && list1.length > 0) {
            result.deleted.push(...list1);
        }
        if (!_.isNil(list1) && !_.isNil(list2)) {
            list1.forEach(elem1 => {
                let found = false;
                for (let i = 0; i < list2.length; i++) {
                    let elem2 = list2[i];
                    if (elem1[key] === elem2[key]) {
                        //check if edited
                        if (!_.isEqual(elem1, elem2)) {
                            result.addedEdited.push(elem2);
                        }
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    result.deleted.push(elem1);
                }
            });
            //Check for added elements
            let addedElement = _.differenceWith(list2, list1, function (element1, element2) {
                return element1[key] === element2[key];
            });
            if (!_.isNil(addedElement)) {
                result.addedEdited.push(...addedElement);
            }
        }
        return result;
    }
}
exports.default = DiffUtil;
//# sourceMappingURL=diffUtil.js.map