"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const xml2js = require("xml2js");
const util = require("util");
const _ = __importStar(require("lodash"));
const diffUtil_1 = __importDefault(require("./diffUtil"));
const nonArayProperties = [
    "description",
    "hasActivationRequired",
    "label",
    "license",
    "userLicense",
    "$",
    "fullName"
];
const parser = new xml2js.Parser({
    explicitArray: false,
    valueProcessors: [
        function (name) {
            if (name === "true")
                name = true;
            if (name === "false")
                name = false;
            return name;
        }
    ]
});
class PermsetDiff {
    constructor(debugFlag) {
        this.debugFlag = debugFlag;
    }
    static async generatePermissionsetXml(permissionsetXml1, permissionsetXml2, outputFilePath) {
        const parseString = util.promisify(parser.parseString);
        let parseResult = await parseString(permissionsetXml1);
        let permsetObj1 = parseResult.PermissionSet;
        parseResult = await parseString(permissionsetXml2);
        let permsetObj2 = parseResult.PermissionSet;
        let newPermsetObj = {};
        newPermsetObj.label = permsetObj2.label;
        if (!_.isNil(permsetObj2.description)) {
            newPermsetObj.description = permsetObj2.description;
        }
        if (!_.isNil(permsetObj2.license)) {
            newPermsetObj.license = permsetObj2.license;
        }
        if (permsetObj2.hasActivationRequired) {
            newPermsetObj.hasActivationRequired = permsetObj2.hasActivationRequired;
        }
        newPermsetObj.applicationVisibilities = diffUtil_1.default.getChangedOrAdded(permsetObj1.applicationVisibilities, permsetObj2.applicationVisibilities, "application").addedEdited;
        newPermsetObj.classAccesses = diffUtil_1.default.getChangedOrAdded(permsetObj1.classAccesses, permsetObj2.classAccesses, "apexClass").addedEdited;
        newPermsetObj.customPermissions = diffUtil_1.default.getChangedOrAdded(permsetObj1.customPermissions, permsetObj2.customPermissions, "name").addedEdited;
        newPermsetObj.externalDataSourceAccesses = diffUtil_1.default.getChangedOrAdded(permsetObj1.externalDataSourceAccesses, permsetObj2.externalDataSourceAccesses, "externalDataSource").addedEdited;
        newPermsetObj.fieldPermissions = diffUtil_1.default.getChangedOrAdded(permsetObj1.fieldPermissions, permsetObj2.fieldPermissions, "field").addedEdited;
        newPermsetObj.objectPermissions = diffUtil_1.default.getChangedOrAdded(permsetObj1.objectPermissions, permsetObj2.objectPermissions, "object").addedEdited;
        newPermsetObj.pageAccesses = diffUtil_1.default.getChangedOrAdded(permsetObj1.pageAccesses, permsetObj2.pageAccesses, "apexPage").addedEdited;
        newPermsetObj.recordTypeVisibilities = diffUtil_1.default.getChangedOrAdded(permsetObj1.recordTypeVisibilities, permsetObj2.recordTypeVisibilities, "recordType").addedEdited;
        newPermsetObj.tabSettings = diffUtil_1.default.getChangedOrAdded(permsetObj1.tabSettings, permsetObj2.tabSettings, "tab").addedEdited;
        newPermsetObj.userPermissions = diffUtil_1.default.getChangedOrAdded(permsetObj1.userPermissions, permsetObj2.userPermissions, "name").addedEdited;
        await PermsetDiff.writePermset(newPermsetObj, outputFilePath);
    }
    static async writePermset(permsetObj, filePath) {
        //Delete eampty arrays
        for (var key in permsetObj) {
            if (Array.isArray(permsetObj[key])) {
                //All top element must be arays exept non arrayProperties
                if (!nonArayProperties.includes(key) && permsetObj[key].length === 0) {
                    delete permsetObj[key];
                }
            }
        }
        if (permsetObj.label != undefined) {
            var builder = new xml2js.Builder({ rootName: "PermissionSet" });
            permsetObj["$"] = {
                xmlns: "http://soap.sforce.com/2006/04/metadata"
            };
            var xml = builder.buildObject(permsetObj);
            fs.writeFileSync(filePath, xml);
        }
    }
}
exports.default = PermsetDiff;
//# sourceMappingURL=permsetDiff.js.map