"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xml2js = require("xml2js");
const util = require("util");
const _ = __importStar(require("lodash"));
const diffUtil_1 = __importDefault(require("./diffUtil"));
const profileWriter_1 = __importDefault(require("../../../impl/metadata/writer/profileWriter"));
class ProfileDiff {
    static async generateProfileXml(profileXml1, profileXml2, outputFilePath) {
        let profileWriter = new profileWriter_1.default();
        const parser = new xml2js.Parser({ explicitArray: true });
        const parseString = util.promisify(parser.parseString);
        let parseResult = await parseString(profileXml1);
        let profileObj1 = profileWriter.toProfile(parseResult.Profile);
        parseResult = await parseString(profileXml2);
        let profileObj2 = profileWriter.toProfile(parseResult.Profile);
        let newProObj = {
            fullName: profileObj2.fullName,
            applicationVisibilities: [],
            classAccesses: [],
            customPermissions: [],
            externalDataSourceAccesses: [],
            fieldLevelSecurities: [],
            fieldPermissions: [],
            layoutAssignments: [],
            loginHours: [],
            loginIpRanges: [],
            objectPermissions: [],
            pageAccesses: [],
            profileActionOverrides: [],
            recordTypeVisibilities: [],
            tabVisibilities: [],
            userPermissions: []
        };
        if (!_.isNil(profileObj2.description)) {
            newProObj.description = profileObj2.description;
        }
        newProObj.applicationVisibilities = diffUtil_1.default.getChangedOrAdded(profileObj1.applicationVisibilities, profileObj2.applicationVisibilities, "application").addedEdited;
        newProObj.classAccesses = diffUtil_1.default.getChangedOrAdded(profileObj1.classAccesses, profileObj2.classAccesses, "apexClass").addedEdited;
        newProObj.customPermissions = diffUtil_1.default.getChangedOrAdded(profileObj1.customPermissions, profileObj2.customPermissions, "name").addedEdited;
        newProObj.externalDataSourceAccesses = diffUtil_1.default.getChangedOrAdded(profileObj1.externalDataSourceAccesses, profileObj2.externalDataSourceAccesses, "externalDataSource").addedEdited;
        newProObj.fieldLevelSecurities = diffUtil_1.default.getChangedOrAdded(profileObj1.fieldLevelSecurities, profileObj2.fieldLevelSecurities, "field").addedEdited;
        newProObj.fieldPermissions = diffUtil_1.default.getChangedOrAdded(profileObj1.fieldPermissions, profileObj2.fieldPermissions, "field").addedEdited;
        newProObj.loginHours = !_.isEqual(profileObj1.loginHours, profileObj2.loginHours)
            ? profileObj2.loginHours
            : [];
        newProObj.loginIpRanges = !_.isEqual(profileObj1.loginIpRanges, profileObj2.loginIpRanges)
            ? profileObj2.loginIpRanges
            : [];
        newProObj.objectPermissions = diffUtil_1.default.getChangedOrAdded(profileObj1.objectPermissions, profileObj2.objectPermissions, "object").addedEdited;
        newProObj.pageAccesses = diffUtil_1.default.getChangedOrAdded(profileObj1.pageAccesses, profileObj2.pageAccesses, "apexPage").addedEdited;
        newProObj.profileActionOverrides = diffUtil_1.default.getChangedOrAdded(profileObj1.profileActionOverrides, profileObj2.profileActionOverrides, "actionName").addedEdited;
        newProObj.recordTypeVisibilities = diffUtil_1.default.getChangedOrAdded(profileObj1.recordTypeVisibilities, profileObj2.recordTypeVisibilities, "recordType").addedEdited;
        newProObj.tabVisibilities = diffUtil_1.default.getChangedOrAdded(profileObj1.tabVisibilities, profileObj2.tabVisibilities, "tab").addedEdited;
        newProObj.userPermissions = diffUtil_1.default.getChangedOrAdded(profileObj1.userPermissions, profileObj2.userPermissions, "name").addedEdited;
        newProObj.layoutAssignments = this.getChangedOrAddedLayouts(profileObj1.layoutAssignments, profileObj2.layoutAssignments);
        profileWriter.writeProfile(newProObj, outputFilePath);
    }
    static getChangedOrAddedLayouts(list1, list2) {
        let result = [];
        if (_.isNil(list1) && !_.isNil(list2) && list2.length > 0) {
            result.push(...list2);
        }
        if (!_.isNil(list1) && !_.isNil(list2)) {
            list2.forEach(layoutAss2 => {
                let found = false;
                for (let i = 0; i < list1.length; i++) {
                    let layoutAss1 = list1[i];
                    if (layoutAss1.layout === layoutAss2.layout) {
                        //check if edited
                        if (_.isEqual(layoutAss1, layoutAss2)) {
                            found = true;
                            break;
                        }
                    }
                }
                if (!found) {
                    result.push(layoutAss2);
                }
            });
        }
        return result;
    }
}
exports.default = ProfileDiff;
//# sourceMappingURL=profileDiff.js.map